#include "networkwdiget.h"
#include "networkbroadcastdialog.h"

#include <QThread>
#include <QTcpSocket>
#include <QUdpSocket>
#include <QLabel>
#include <QComboBox>
#include <QPushButton>
#include <QMessageBox>
#include <QVBoxLayout>
#include <QLineEdit>
#include <QHostAddress>
#include <QtDebug>
#include <QCoreApplication>
#include <QTime>
#include <QNetworkInterface>

NetworkWdiget::NetworkWdiget(QWidget *parent) : QWidget(parent)
{
    // 单次触发计时器，用于读取网口超时计时
    _timer.setSingleShot(true);
    _timer.setInterval(100);
    connect(&_timer,SIGNAL(timeout()),this,SLOT(slotOnTimeout()));

    _p_socket = new QTcpSocket;
    connect(_p_socket,SIGNAL(connected()),this,SIGNAL(connected()));
    connect(_p_socket,SIGNAL(disconnected()),this,SIGNAL(disconnected()));
    connect(_p_socket,SIGNAL(readyRead()),this,SLOT(doRead()));

    // local ip
    // 获取本机IP地址
    QList<QHostAddress> list = QNetworkInterface::allAddresses();
    QString ipAddr("192.168.43.1");
    foreach(QHostAddress address,list) {
        if(address.protocol() != QAbstractSocket::IPv4Protocol) {
            continue;
        }

        QString addr = address.toString().simplified();
        if(addr.left(3) == "127") {
            continue;
        }

        // 本机上所有IP地址中满足是IPv4协议并且开头不是127的地址则认为是本机IP地址
        ipAddr = addr;

        break;
    }

    // 创建控件
    QLabel *pIPLbl = new QLabel(tr("IP"));
    pIPLbl->setFixedWidth(80);
    _p_ipEdit = new QLineEdit(ipAddr);
    _p_ipEdit->setFixedWidth(130);

    QLabel *pPortLbl = new QLabel(tr("Port"));
    pPortLbl->setFixedWidth(80);
    _p_portEdit = new QLineEdit("2000");
    _p_portEdit->setFixedWidth(130);

    QLabel *pModeLbl = new QLabel(tr("Mode"));
    pModeLbl->setFixedWidth(80);
    _p_modeCbx = new QComboBox;
    _p_modeCbx->addItems(QStringList() << tr("Ascii") << tr("Unicode"));
    _p_modeCbx->setCurrentIndex(0);
    _p_modeCbx->setFixedWidth(130);
    connect(_p_modeCbx,SIGNAL(currentIndexChanged(int)),this,SLOT(slotOnModeChanged()));

    _p_connectBtn = new QPushButton(tr("Connect"));
    _p_connectBtn->setFixedWidth(130);
    connect(_p_connectBtn,SIGNAL(clicked()),this,SLOT(slotOnDoConnect()));

    _p_disconnectBtn = new QPushButton(tr("Disconnect"));
    _p_disconnectBtn->setFixedWidth(130);
    connect(_p_disconnectBtn,SIGNAL(clicked()),this,SLOT(slotOnDoDisconnect()));

    _p_broadcastBtn = new QPushButton(tr("Quick search"));
    _p_broadcastBtn->setFixedWidth(130);
    connect(_p_broadcastBtn,SIGNAL(clicked()),this,SLOT(slotOnSearch()));

    // 控件布局
    QGridLayout *pGlt = new QGridLayout;
    pGlt->addWidget(pIPLbl,0,0,1,1);
    pGlt->addWidget(_p_ipEdit,0,1,1,1);

    pGlt->addWidget(pPortLbl,1,0,1,1);
    pGlt->addWidget(_p_portEdit,1,1,1,1);

    pGlt->addWidget(pModeLbl,2,0,1,1);
    pGlt->addWidget(_p_modeCbx,2,1,1,1);

    pGlt->addWidget(_p_connectBtn,3,1,1,1);
    pGlt->addWidget(_p_disconnectBtn,4,1,1,1);
    pGlt->addWidget(_p_broadcastBtn,5,1,1,1);

    setLayout(pGlt);
}

NetworkWdiget::~NetworkWdiget()
{
    delete _p_socket;
}

bool NetworkWdiget::isAvailable() const
{
    return isVisible();
}

bool NetworkWdiget::canRead() const
{
    return (_p_socket->state() == QTcpSocket::ConnectedState);
}

bool NetworkWdiget::canWrite() const
{
    return (_p_socket->state() == QTcpSocket::ConnectedState);
}

// 发送数据到板卡
int NetworkWdiget::write(const QByteArray &bytes)
{
    if(_p_socket->state() != QTcpSocket::ConnectedState) {
        return -1;
    }

    return _p_socket->write(bytes);
}

// 连接板卡
void NetworkWdiget::slotOnDoConnect()
{
    // 先断开旧连接
    _p_socket->disconnectFromHost();
    if((_p_socket->state() != QAbstractSocket::UnconnectedState) && !_p_socket->waitForDisconnected(2000)) {
        QMessageBox::information(this,tr("Tip"),tr("Disconnect failed."));
        return;
    }

    // 再连接板卡
    _p_socket->connectToHost(QHostAddress(_p_ipEdit->text()),_p_portEdit->text().toUShort());
    if(_p_socket->waitForConnected(2000)) {
        QMessageBox::information(this,tr("Tip"),tr("Connect ok."));
    } else {
        QMessageBox::information(this,tr("Tip"),tr("Connect failed."));
    }

    updateState();
}

// 与板卡断开连接
void NetworkWdiget::slotOnDoDisconnect()
{
    _p_socket->disconnectFromHost();
    _p_socket->waitForDisconnected(2000);
    if(_p_socket->state() != QAbstractSocket::UnconnectedState) {
        QMessageBox::information(this,tr("Tip"),tr("Disconnect failed."));
    } else {
        QMessageBox::information(this,tr("Tip"),tr("Disconnect ok."));
    }

    updateState();
}

void NetworkWdiget::slotOnSearch()
{

    // 创建广播搜索板卡对话框，并执行对话框
    NetworkBroadcastDialog dlg;

    // 未找到板卡，则退出
    if(!dlg.exec()) {
        //QMessageBox::information(this,tr("Tip"),"未找到可用设备，请检查静态板卡版本是否3.1.63.53或以上，\n然后检查电脑或板卡网络设置");
        QMessageBox::information(this,tr("Tip"),tr("No found device,please check board version is 3.1.63.53 or higher,\nand check computer or board network setting."));
        return;
    }

    // 找到板卡，将通过广播搜索板卡对话框搜索板卡时读取的板卡IP地址，板卡端口号，板卡通讯模式，设置到界面上的控件上
    _p_ipEdit->setText(dlg.ipAddr());
    _p_portEdit->setText(QString::number(dlg.port()));
    _p_modeCbx->setCurrentIndex(dlg.isUnicode() ? 1 : 0);

    // 弹窗提示是否连接搜索到的板卡，如果选择是，则连接板卡
    if(QMessageBox::information(this,tr("Tip"),tr("Already found device,continue to connect?"),QMessageBox::Yes | QMessageBox::No,QMessageBox::Yes) == QMessageBox::Yes) {
        slotOnDoConnect();
    }
}

void NetworkWdiget::slotOnModeChanged()
{
    _protocol = _p_modeCbx->currentIndex();
}

void NetworkWdiget::slotOnTimeout()
{
    if(!_readBytes.isEmpty()) {
        emit readyRead();
    }
}

void NetworkWdiget::updateState()
{
    // 根据连接状态修改控件使能状态
    bool isConnected = _p_socket->state() == QTcpSocket::ConnectedState;
    _p_ipEdit->setEnabled(!isConnected);
    _p_portEdit->setEnabled(!isConnected);
    _p_modeCbx->setEnabled(!isConnected);
    _p_connectBtn->setEnabled(!isConnected);
    _p_broadcastBtn->setEnabled(!isConnected);
}

// 读取网口接收到的数据
void NetworkWdiget::doRead()
{
    if(!_timer.isActive()) {
        // 计时器未在运行时网口接收到数据，则认为是新的一帧数据
        _readBytes = _p_socket->readAll();
    } else {
        // 计时器在运行时网口接收到数据，则认为是上一帧数据的剩余部分
        _readBytes += _p_socket->readAll();
    }

    // 每次读取网口接收到的数据后，计时器开始计时，当下次再读取网口接收到的数据时，对计时器状态进行判断
    // 如果下次再读取网口接收到的数据时，计时器仍在运行，则表示计时器间隔时间还没过完，认为新读取的数据是上一帧数据的剩余部分
    // 如果下次再读取网口接收到的数据时，计时器已经结束运行，则表示计时器间隔时间已经过完，认为新读取的数据是新一帧数据
    _timer.start();
}
